@@ -202,6 +202,7 @@ vardomh <- function(Y, H, PSU, w_final,
   }
 
   # Dom
+  namesDom <- NULL  
   if (!is.null(Dom)) {
     Dom <- data.table(Dom)
     if (any(duplicated(names(Dom)))) 
@@ -211,11 +212,12 @@ vardomh <- function(Y, H, PSU, w_final,
     if (any(is.na(Dom))) stop("'Dom' has unknown values")
     if (is.null(names(Dom))) stop("'Dom' must be colnames")
     Dom <- Dom[, lapply(.SD, as.character), .SDcols = names(Dom)]
+    namesDom <- names(Dom)
   }
   
   # Z
   if (!is.null(Z)) {
-    Z <- data.table(Z, check.names = T)
+    Z <- data.table(Z, check.names = TRUE)
     if (nrow(Z) != n) stop("'Z' and 'Y' must be equal row count")
     if (ncol(Z) != m) stop("'Z' and 'Y' must be equal column count")
     if (!all(sapply(Z, is.numeric))) stop("'Z' must be numeric values")
@@ -265,7 +267,7 @@ vardomh <- function(Y, H, PSU, w_final,
 
   # X
   if (!is.null(X)) {
-    X <- data.table(X, check.names=T)
+    X <- data.table(X, check.names=TRUE)
     if (nrow(X) != nrow(X_ID_household)) stop("'X' and 'X_ID_household' have different row count")
   }
 
@@ -280,7 +282,7 @@ vardomh <- function(Y, H, PSU, w_final,
 
   # X
   if (!is.null(X)) {
-       X1 <- data.table(X, check.names=T)
+       X1 <- data.table(X, check.names=TRUE)
        nX1 <- names(X1)
        ind_gr1 <- copy(ind_gr) 
        if (!is.null(periodX)) ind_gr1 <- data.table(periodX, ind_gr1, check.names=TRUE)
@@ -294,7 +296,7 @@ vardomh <- function(Y, H, PSU, w_final,
 
   # g
   if (!is.null(X)) {
-    if (is.null(class(g))| class(g)=="function") stop("'g' must be numerical")
+    if (is.null(class(g)) | all(class(g)=="function")) stop("'g' must be numerical")
     g <- data.frame(g)
     if (nrow(g) != nrow(X)) stop("'g' length must be equal with 'X' row count")
     if (ncol(g) != 1) stop("'g' must be 1 column data.frame, matrix, data.table")
@@ -306,7 +308,7 @@ vardomh <- function(Y, H, PSU, w_final,
     
   # q
   if (!is.null(X)) {
-    if (is.null(class(q))| class(q)=="function") stop("'q' must be numerical")
+    if (is.null(class(q)) | all(class(q)=="function")) stop("'q' must be numerical")
     q <- data.frame(q)
     if (nrow(q) != nrow(X)) stop("'q' length must be equal with 'X' row count")
     if (ncol(q) != 1) stop("'q' must be 1 column data.frame, matrix, data.table")
@@ -330,15 +332,15 @@ vardomh <- function(Y, H, PSU, w_final,
                                                          sum(as.integer(abs(x)> .Machine$double.eps))),
                                                          .SDcols = names(Y1)]
 
-  sample_size <- pop_size <- NULL
-  nhs <- data.table(sample_size=1, pop_size=w_final)
+  respondent_count <- sample_size <- pop_size <- NULL
+  nhs <- data.table(respondent_count=1, pop_size=w_final)
   if (!is.null(period)) nhs <- data.table(period, nhs)
   if (!is.null(Dom)) nhs <- data.table(Dom, nhs)
-  if (!is.null(c(Dom, period))) {nhs <- nhs[, lapply(.SD, sum, na.rm=T),
+  if (!is.null(c(Dom, period))) {nhs <- nhs[, lapply(.SD, sum, na.rm=TRUE),
                                                        keyby=eval(names(nhs)[0:1-ncol(nhs)]),
-                                                      .SDcols=c("sample_size", "pop_size")]
-                          } else nhs <- nhs[, lapply(.SD, sum, na.rm=T),
-                                                     .SDcols=c("sample_size", "pop_size")]
+                                                      .SDcols=c("respondent_count", "pop_size")]
+                          } else nhs <- nhs[, lapply(.SD, sum, na.rm=TRUE),
+                                                     .SDcols=c("respondent_count", "pop_size")]
 
   # Design weights
   if (!is.null(X)) {
@@ -357,7 +359,7 @@ vardomh <- function(Y, H, PSU, w_final,
   cv <- absolute_margin_of_error <- NULL
   relative_margin_of_error <- CI_lower <- NULL
   CI_upper <- variable <- variableZ <- .SD <- NULL
-  deff_sam <- deff_est <- deff <- NULL
+  deff_sam <- deff_est <- deff <- n_eff <- NULL
   
   aH <- names(H)
   idper <- id
@@ -391,19 +393,19 @@ vardomh <- function(Y, H, PSU, w_final,
   YY <- data.table(idper, ID_household, H, PSU, w_final, Y2)
   YYa <- data.table(idper, ID_household, H, PSU, w_design, Y2a)
   
-  YY2 <- YY[, lapply(.SD, sum, na.rm = T), by = c(names(YY)[c(2:(5+np))]), .SDcols = names(YY)[-(1:(5+np))]]
-  YY2a <- YYa[, lapply(.SD, sum, na.rm = T), by = c(names(YYa)[c(2:(5+np))]), .SDcols = names(YYa)[-(1:(5+np))]]
-  Y3 <- YY2[, c(-(1:(4+np))), with=F]
-  Y3a <- YY2a[, c(-(1:(4+np))), with=F]
+  YY2 <- YY[, lapply(.SD, sum, na.rm=TRUE), by=c(names(YY)[c(2:(5+np))]), .SDcols=names(YY)[-(1:(5+np))]]
+  YY2a <- YYa[, lapply(.SD, sum, na.rm=TRUE), by=c(names(YYa)[c(2:(5+np))]), .SDcols=names(YYa)[-(1:(5+np))]]
+  Y3 <- YY2[, c(-(1:(4+np))), with=FALSE]
+  Y3a <- YY2a[, c(-(1:(4+np))), with=FALSE]
 
   period <- NULL
-  if (np>0) period <- YY2[, c(1:np), with=F]
+  if (np>0) period <- YY2[, c(1:np), with=FALSE]
 
-  IDh <- YY2[, np+1, with=F]
-  H <- YY2[, np+2, with=F]
+  IDh <- YY2[, np+1, with=FALSE]
+  H <- YY2[, np+2, with=FALSE]
   setnames(H, names(H), aH)
 
-  PSU <- YY2[, np+3, with=F]
+  PSU <- YY2[, np+3, with=FALSE]
   setnames(PSU, names(PSU), aPSU)
 
   w_final2 <- YY2[[np+4]]
@@ -441,7 +443,7 @@ vardomh <- function(Y, H, PSU, w_final,
   all_result <- var_est
 
   n_nonzero <- transpos(n_nonzero, is.null(period), "n_nonzero", names(period))
-  all_result <- merge(all_result, n_nonzero, all=T)
+  all_result <- merge(all_result, n_nonzero, all=TRUE)
  
   # Variance of HT estimator under current design
   var_cur_HT <- variance_est(Y=Y3a, H=H, PSU=PSU, w_final=w_design2,
@@ -490,9 +492,9 @@ vardomh <- function(Y, H, PSU, w_final,
   Y_nov <- Z_nov <- .SD <- NULL
 
   hY <- data.table(Y1*w_final)
-  if (is.null(period)) { Y_nov <- hY[, lapply(.SD, sum, na.rm = T), .SDcols = names(Y1)]
+  if (is.null(period)) { Y_nov <- hY[, lapply(.SD, sum, na.rm=TRUE), .SDcols = names(Y1)]
                 } else { hY <- data.table(period, hY)
-                         Y_nov <- hY[, lapply(.SD, sum, na.rm = T), keyby=names(period), .SDcols = names(Y1)]
+                         Y_nov <- hY[, lapply(.SD, sum, na.rm=TRUE), keyby=names(period), .SDcols = names(Y1)]
                        }
   Y_nov <- transpos(Y_nov, is.null(period), "Y_nov", names(period))
   all_result <- merge(all_result, Y_nov)
@@ -504,9 +506,9 @@ vardomh <- function(Y, H, PSU, w_final,
          all_result <- merge(all_result, YZnames)
          
          hZ <- data.table(Z1*w_final)
-         if (is.null(period)) { Z_nov <- hZ[, lapply(.SD, sum, na.rm = T), .SDcols = names(Z1)]
+         if (is.null(period)) { Z_nov <- hZ[, lapply(.SD, sum, na.rm=TRUE), .SDcols = names(Z1)]
                        } else { hZ <- data.table(period, hZ)
-                                Z_nov <- hZ[, lapply(.SD, sum, na.rm = T), keyby=names(period), .SDcols = names(Z1)]
+                                Z_nov <- hZ[, lapply(.SD, sum, na.rm=TRUE), keyby=names(period), .SDcols = names(Z1)]
                               }
          Z_nov <- transpos(Z_nov, is.null(period), "Z_nov", names(period), "variableDZ")
          setkeyv(all_result, "variableDZ")
@@ -566,34 +568,36 @@ vardomh <- function(Y, H, PSU, w_final,
   nosr <- nosr[, lapply(nosr, as.character)]
   setnames(nosr, names(nosr)[2], "variable")
 
+
+ namesDom1 <- namesDom
   if (!is.null(Dom)) {
-       setnames(nosr, names(nosr)[3:ncol(nosr)], names(Dom))
-       nosr1 <- nosr[, lapply(names(Dom), function(x) {substring(get(x), nchar(x)+2, nchar(get(x)))})] 
-       nosr1 <- nosr1[, lapply(names(nosr1), function(x) str_replace_all(get(x), "[.]", " "))]
-       setnames(nosr1, names(nosr1), names(Dom))
-       setnames(nosr, names(Dom), paste0(names(Dom), "old"))
-       nosr <- data.table(nosr, nosr1)
+       setnames(nosr, names(nosr)[3:ncol(nosr)], paste0(namesDom, "_new"))
+       nhs[, (paste0(namesDom, "_new")):=lapply(namesDom, function(x) make.names(paste0(x,".", get(x))))]
+       namesDom1 <- paste0(namesDom, "_new")
     }
 
   setkeyv(nosr, "variableD")
   setkeyv(all_result, "variableD")
   all_result <- merge(nosr, all_result)
-  nosr <- nosr1 <- NULL
-
-  if (!is.null(all_result$Z_nov)) all_result[, variable:=paste("R", get("variable"), sep="__", get("variableZ"))]
-  setkeyv(all_result, c(names(Dom), names(period)))
-
-  if (!is.null(c(Dom, period))) { all_result <- merge(all_result, nhs, all=T)
-                         } else { all_result[, sample_size:=nhs$sample_size]
-                                  all_result[, pop_size:=nhs$pop_size]} 
+  namesDom <- nosr <- NULL
   
-  variab <- c("sample_size", "n_nonzero", "pop_size", "estim", "var", "se", 
+  if (!is.null(all_result$Z_nov)) { 
+       all_result[, variable:=paste("R", get("variable"), sep="__", get("variableDZ"))] }
+  setkeyv(all_result, c(namesDom1, names(period)))
+  setkeyv(nhs, c(namesDom1, names(period)))
+
+  if (!is.null(c(Dom, period))) { all_result <- merge(all_result, nhs, all=TRUE)
+                         } else { all_result[, respondent_count:=nhs$respondent_count]
+                                  all_result[, pop_size:=nhs$pop_size]} 
+
+  all_result[, n_eff:=deff/respondent_count]
+  variab <- c("respondent_count", "n_nonzero", "pop_size", "estim", "var", "se", 
               "rse", "cv", "absolute_margin_of_error", "relative_margin_of_error",
               "CI_lower", "CI_upper", "var_srs_HT",  "var_cur_HT", 
-              "var_srs_ca", "deff_sam", "deff_est", "deff")
+              "var_srs_ca", "deff_sam", "deff_est", "deff", "n_eff")
 
   setkeyv(all_result, c("nr_names", names(Dom), names(period)))
-  all_result <- all_result[, c("variable", names(Dom), names(period), variab), with=F]
+  all_result <- all_result[, c("variable", names(Dom), names(period), variab), with=FALSE]
   
   list(lin_out = linratio_outp,
        res_out = res_outp,
