@@ -107,7 +107,7 @@ variance_est <- function(Y, H, PSU, w_final, N_h=NULL, fh_zero=FALSE, PSU_level=
     Nh <- data.table(H, w_final)
     if (!is.null(period)) Nh <- data.table(period, Nh)
     setkeyv(Nh, names(Nh)[c(1:(1+np))])
-    N_h <- Nh[, list(N_h = sum(w_final, na.rm = T)), keyby=c(names(Nh)[1:(1+np)])]
+    N_h <- Nh[, list(N_h = sum(w_final, na.rm = TRUE)), keyby=c(names(Nh)[1:(1+np)])]
   }
   pH <- NULL  
 
@@ -117,13 +117,13 @@ variance_est <- function(Y, H, PSU, w_final, N_h=NULL, fh_zero=FALSE, PSU_level=
   .SD <- .N <- NULL
   hpY <- data.table(H, PSU, Y*w_final)
   if (!is.null(period)) hpY <- data.table(period, hpY)
-  z_hi <- hpY[, lapply(.SD, sum, na.rm = T), 
+  z_hi <- hpY[, lapply(.SD, sum, na.rm=TRUE), 
                        keyby = c(names(hpY)[1:(2+np)]),
                        .SDcols = names(hpY)[-(1:(2+np))]]
   setkeyv(z_hi, names(z_hi)[c(1:(1+np))]) 
 
   # n_h
-  n_h <- data.table(z_hi[, c(1:(1+np)), with=F])
+  n_h <- data.table(z_hi[, c(1:(1+np)), with=FALSE])
   n_h <- n_h[,.N, keyby=c(names(n_h)[1:(1+np)])]
   setnames(n_h, "N", "n_h")
 
@@ -132,17 +132,17 @@ variance_est <- function(Y, H, PSU, w_final, N_h=NULL, fh_zero=FALSE, PSU_level=
     print("There is stratas, where n_h == 1")
     print("Not possible to estimate the variance in these stratas!")
     print("At these stratas estimation of variance was not calculated")
-    nh <- n_h[, 2+np, with=F]==1 
+    nh <- n_h[, 2+np, with=FALSE]==1 
     print(data.frame(n_h)[nh,])
   }
   
   # var_z_hi
-  var_z_hi <- z_hi[, lapply(.SD, var, na.rm = F), keyby = c(names(z_hi)[1:(1+np)]),.SDcols = names(z_hi)[-(1:(2+np))]]
+  var_z_hi <- z_hi[, lapply(.SD, var, na.rm=FALSE), keyby = c(names(z_hi)[1:(1+np)]),.SDcols = names(z_hi)[-(1:(2+np))]]
   
   # f_h
-  F_h <- merge(N_h, n_h, by = names(hpY)[c(1:(1+np))], sort = T)
+  F_h <- merge(N_h, n_h, by = names(hpY)[c(1:(1+np))], sort=TRUE)
   F_h[, f_h:=n_h/N_h]
-  f_h <- F_h[,"f_h", with=F]
+  f_h <- F_h[,"f_h", with=FALSE]
 
   if (any(f_h > 1)) {    
      print("There is stratas, where f_h > 1")
@@ -154,17 +154,17 @@ variance_est <- function(Y, H, PSU, w_final, N_h=NULL, fh_zero=FALSE, PSU_level=
   # fh1
   if (!(PSU_level)) {
      n_h1 <- NULL
-     fh1 <- data.table(hpY[, c(1:(1+np)), with=F], w_final)
+     fh1 <- data.table(hpY[, c(1:(1+np)), with=FALSE], w_final)
      fh1[, n_h1:=1]
-     fh1 <- fh1[, lapply(.SD, sum, na.rm=T), keyby = c(names(fh1)[c(1:(1+np))]), .SDcols = c("n_h1", "w_final")]
+     fh1 <- fh1[, lapply(.SD, sum, na.rm=TRUE), keyby = c(names(fh1)[c(1:(1+np))]), .SDcols = c("n_h1", "w_final")]
      fh1[, fh1:=n_h1/w_final]
-     f_h <- fh1[,"fh1", with=F]
+     f_h <- fh1[,"fh1", with=FALSE]
    }
 
   # var_h
-  var_h <- data.table(matrix((1 - f_h*(1-fh_zero)) * n_h$n_h) * var_z_hi[, -c(1:(1+np)), with=F])
+  var_h <- data.table(matrix((1 - f_h*(1-fh_zero)) * n_h$n_h) * var_z_hi[, -c(1:(1+np)), with=FALSE])
   var_h2 <- copy(var_h)
-  if (np>0) var_h2 <- data.table(var_z_hi[, c(1:np), with=F], var_h)
+  if (np>0) var_h2 <- data.table(var_z_hi[, c(1:np), with=FALSE], var_h)
 
   # Variance_est 
 
